#ifndef __ClassRWeakPtr_Header
#define __ClassRWeakPtr_Header

#include <iostream>
using std::cout;
#include <memory> // wegen shared_ptr
using std::weak_ptr;
#include <vector>
using std::vector;

class R2 {
public:
   R2(int id = 9) : m_id(id) {
      cout << "+R" << id << " ";
   }
   ~R2() {
      cout << "-R" << m_id << " ";
   }
   void addFr(weak_ptr<R2> f) {
      myFr.push_back(f);
   }
   void printFriends() const;
   void printFriendsBad() const;
private:
   vector<weak_ptr<R2>> myFr;
   int m_id;
};

inline
void R2::printFriends() const {
   cout << "friends: ";
   for (auto iter : myFr) {
      shared_ptr<R2> sp = iter.lock();
      if (sp) {
         cout << sp->m_id << " ";
      }
   } // for iter
}


inline
void R2::printFriendsBad() const {
   cout << "friends: ";
   for (auto iter : myFr) {
      shared_ptr<R2> sp = iter.lock();
      // here we may get a crash because conversion may
      // fail, because object already destroyed
      cout << sp->m_id << " ";
   } // for iter
}

#ifdef WITHOUT_AUTO_USAGE
inline
void R2::printFriends() const {
   cout << "friends: ";
   for (vector<weak_ptr<R2>>::const_iterator iter = myFr.begin();
        iter != myFr.end(); ++iter) {
      shared_ptr<R2> sp = iter->lock();
      if (sp) {
         cout << sp->m_id << " ";
      }
   } // for iter
}
#endif

#endif /* __ClassRWeakPtr_Header */

